/******************************************************************************
* Copyright (C) 2008 - Ikaro Games   www.ikarogames.com                       *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                             *
*                                                                             *
*       generated by dia/DAOcodegen.py                                        *
*       Version: 1.11                                                          *
******************************************************************************/

#include "CDAOFactorySQLite.h"
#include "../../../../utils/CLog.h"

CDAOFactorySQLite::CDAOFactorySQLite(std::string file)
{
    CLog::getInstance()->info("Path: %s", file.c_str());
    if( sqlite3_open(file.c_str(), &m_database )!=SQLITE_OK ){
        sqlite3_close(m_database);
        m_database = NULL;
        CLog::getInstance()->exception("Can't open database file: \"%s\" --> \"%s\"", file.c_str(), sqlite3_errmsg(m_database));
    }

    m_PfGoalsDAOSQLite = new CPfGoalsDAOSQLite(m_database);
    m_PfGameStatesDAOSQLite = new CPfGameStatesDAOSQLite(m_database);
    m_PfTeamsDAOSQLite = new CPfTeamsDAOSQLite(m_database);
    m_PfMatchesDAOSQLite = new CPfMatchesDAOSQLite(m_database);
    m_PfGameOptionsDAOSQLite = new CPfGameOptionsDAOSQLite(m_database);

    CLog::getInstance()->info("SQLite Database open: <-- \"%s\"", file.c_str());
}

CDAOFactorySQLite::~CDAOFactorySQLite()
{
    delete m_PfGoalsDAOSQLite;
    delete m_PfGameStatesDAOSQLite;
    delete m_PfTeamsDAOSQLite;
    delete m_PfMatchesDAOSQLite;
    delete m_PfGameOptionsDAOSQLite;

    sqlite3_close(m_database);
    CLog::getInstance()->info("SQLite Database closed");
}

bool CDAOFactorySQLite::createSchema()
{
    std::string sql = "";
    sql += "BEGIN TRANSACTION;";
    sql += "CREATE TABLE PF_GOALS(X_FK_TEAM_SCORER INTEGER,X_GOAL INTEGER PRIMARY KEY AUTOINCREMENT,N_MINUTE INTEGER,X_FK_MATCH INTEGER,L_OWN_GOAL TEXT);";
    sql += "CREATE TABLE PF_GAME_STATES(S_STATE TEXT,X_STATE INTEGER PRIMARY KEY AUTOINCREMENT,S_VALUE TEXT);";
    sql += "CREATE TABLE PF_TEAMS(X_TEAM INTEGER PRIMARY KEY AUTOINCREMENT,S_TEAM TEXT);";
    sql += "CREATE TABLE PF_MATCHES(D_DATE TEXT,X_FK_TEAM_HOME INTEGER,X_MATCH INTEGER PRIMARY KEY AUTOINCREMENT,X_FK_TEAM_AWAY INTEGER);";
    sql += "CREATE TABLE PF_GAME_OPTIONS(S_CATEGORY TEXT,X_OPTION INTEGER PRIMARY KEY AUTOINCREMENT,S_VALUE TEXT,S_ATTRIBUTE TEXT);";
    sql += "CREATE INDEX PF_GOALS_X_FK_TEAM_SCORER ON PF_GOALS (X_FK_TEAM_SCORER);";
    sql += "CREATE INDEX PF_GOALS_X_GOAL ON PF_GOALS (X_GOAL);";
    sql += "CREATE INDEX PF_GOALS_X_FK_MATCH ON PF_GOALS (X_FK_MATCH);";
    sql += "CREATE INDEX PF_GAME_STATES_X_STATE ON PF_GAME_STATES (X_STATE);";
    sql += "CREATE INDEX PF_TEAMS_X_TEAM ON PF_TEAMS (X_TEAM);";
    sql += "CREATE INDEX PF_MATCHES_X_FK_TEAM_HOME ON PF_MATCHES (X_FK_TEAM_HOME);";
    sql += "CREATE INDEX PF_MATCHES_X_MATCH ON PF_MATCHES (X_MATCH);";
    sql += "CREATE INDEX PF_MATCHES_X_FK_TEAM_AWAY ON PF_MATCHES (X_FK_TEAM_AWAY);";
    sql += "CREATE INDEX PF_GAME_OPTIONS_X_OPTION ON PF_GAME_OPTIONS (X_OPTION);";
    sql += "COMMIT;";

    char *msgError = NULL;
    bool correct = true;
    if( sqlite3_exec(m_database, sql.c_str(), NULL, NULL, &msgError)!=SQLITE_OK ){
        CLog::getInstance()->error("Error in SQL: \"%s\" --> \"%s\"", sql.c_str(), msgError);
        sqlite3_free(msgError);
        correct = false;
    }
    return correct;
}

bool CDAOFactorySQLite::beginTransaction()
{
    char *msgError = NULL;
    bool correct = true;
    if( sqlite3_exec(m_database, "BEGIN TRANSACTION;", NULL, NULL, &msgError)!=SQLITE_OK ){
        CLog::getInstance()->error("Error in 'BEGIN TRANSACTION;': \"%s\"", msgError);
        sqlite3_free(msgError);
        correct = false;
    }
    return correct;
}

bool CDAOFactorySQLite::commit()
{
    char *msgError = NULL;
    bool correct = true;
    if( sqlite3_exec(m_database, "COMMIT;", NULL, NULL, &msgError)!=SQLITE_OK ){
        CLog::getInstance()->error("Error in 'COMMIT;': \"%s\"", msgError);
        sqlite3_free(msgError);
        correct = false;
    }
    return correct;
}

bool CDAOFactorySQLite::rollback()
{
    char *msgError = NULL;
    bool correct = true;
    if( sqlite3_exec(m_database, "ROLLBACK;", NULL, NULL, &msgError)!=SQLITE_OK ){
        CLog::getInstance()->error("Error in 'ROLLBACK;': \"%s\"", msgError);
        sqlite3_free(msgError);
        correct = false;
    }
    return correct;
}

IPfGoalsDAO* CDAOFactorySQLite::getIPfGoalsDAO()
{
    return m_PfGoalsDAOSQLite;
}

IPfGameStatesDAO* CDAOFactorySQLite::getIPfGameStatesDAO()
{
    return m_PfGameStatesDAOSQLite;
}

IPfTeamsDAO* CDAOFactorySQLite::getIPfTeamsDAO()
{
    return m_PfTeamsDAOSQLite;
}

IPfMatchesDAO* CDAOFactorySQLite::getIPfMatchesDAO()
{
    return m_PfMatchesDAOSQLite;
}

IPfGameOptionsDAO* CDAOFactorySQLite::getIPfGameOptionsDAO()
{
    return m_PfGameOptionsDAOSQLite;
}

